classdef vector3d < dynOption
  
  properties 
    x = []; % x coordinate
    y = []; % y coordinate
    z = []; % z coordinate
    antipodal = false;
  end
    
  properties (Dependent = true)
    theta   % polar angle
    rho     % azimuth angle
    resolution % mean distance between the points on the sphere
    xyz
  end
  
  methods
    
    function v = vector3d(x,y,z,varargin)
      % Constructor
      %
      % Syntax
      %   v = vector3d(x,y,z)
      %
      % Input
      %  x,y,z - cart. coordinates
      %
      % Flags
      %   antipodal - consider vector as an axis and not as an direction
      %
      % See also
      % AxialDirectional

      if nargin == 0
      elseif nargin <= 2
        if isa(x,'vector3d') % copy-constructor
          v = x;
          return
        elseif isa(x,'double')
          if all(size(x) == [1,3])
            x = x.';
          end
          v.x = x(1,:);
          v.y = x(2,:);
          v.z = x(3,:);
        else
          error('wrong type of argument');
        end
        
      elseif nargin >=3 && isnumeric(x) && isnumeric(y) && isnumeric(z)
        
        v.x = x;
        v.y = y;
        v.z = z;
        
      elseif ischar(x)
        
        if strcmp(x,'polar')
          
          sy = sin(y);
          v.x = sy .* cos(z);
          v.y = sy .* sin(z);
          v.z = cos(y);
          
        else
          
          theta = get_option([{x,y,z},varargin],{'theta','azimuth'});
          rho = get_option([{x,y,z},varargin],{'rho','polar'});
          
          v.x = sin(theta).*cos(rho);
          v.y = sin(theta).*sin(rho);
          v.z = cos(theta);
          
        end
                
      end

      % ----------- check for equal size ------------------------
      if numel(v.x) ~= numel(v.y) || (numel(v.x) ~= numel(v.z))
  
        % find non singular size
        if numel(v.x) > 1
          s = size(v.x);
        elseif numel(v.y) > 1
          s = size(v.y);
        else
          s = size(v.z);
        end
  
        % try to correct
        if numel(v.x) == 1, v.x = repmat(v.x,s);end
        if numel(v.y) == 1, v.y = repmat(v.y,s);end
        if numel(v.z) == 1, v.z = repmat(v.z,s);end
  
        % check again
        if numel(v.x) ~= numel(v.y) || (numel(v.x) ~= numel(v.z))
          error('MTEX:Vector3d','Coordinates have different size.');
        end
      end

      % ------------------ options ------------------------------
      
      % antipodal
      v.antipodal = check_option(varargin,'antipodal');
      
      % resolution
      if check_option(varargin,'resolution')
        v = v.setOption('resolution',get_option(varargin,'resolution'));
      end
      
      % normalize
      if nargin > 3 && check_option(varargin,'normalize'), v = v ./ norm(v); end
      
    end
  
    function n = numArgumentsFromSubscript(varargin)
      n = 0;
    end
    
    function rho = get.rho(v)
      try
        rho = v.opt.rho;
      catch
        rho = atan2(v.y,v.x);
      end
    end
    
    function theta = get.theta(v)
      try
        theta = v.opt.theta;
      catch
        theta = acos(v.z./v.norm);
      end
    end
    
    function xyz = get.xyz(v)
      
      xyz = [v.x(:),v.y(:),v.z(:)];
      
    end
    
    function res = get.resolution(v)
      
      if v.isOption('resolution')
        res = v.getOption('resolution');
      elseif length(v) <= 4
        res = 2*pi;
      elseif length(v) > 50000
        res = 1*pi;
      else
        try
          a = calcVoronoiArea(v);
          res = sqrt(median(a));
          assert(res>0);
        catch
            res = 2*pi;
        end        
      end
    end
    
    function v = set.resolution(v,res)
      
      v = v.setOption('resolution',res);
      
    end
  end
  
  methods (Static = true)
    
    function v = nan(varargin)
      x = nan(varargin{:});
      v = vector3d(x,x,x);
    end
    
    function v = ones(varargin)
      x = ones(varargin{:});
      v = vector3d(x,x,x);
    end
    
    function v = zeros(varargin)
      x = zeros(varargin{:});
      v = vector3d(x,x,x);
    end
        
    function v = X(varargin)
      % the vector (1,0,0)
      %
      % Syntax
      %   x = vector3d.X % returns a single vector (1,0,0)
      %   x = vector3d.X(3,1) % returns 3 vectors (1,0,0)
      
      x = ones(varargin{:});
      v = vector3d(x,0,0);
    end
    
    function v = Y(varargin)
      x = ones(varargin{:});
      v = vector3d(0,x,0);
    end
    
    function v = Z(varargin)
      x = ones(varargin{:});
      v = vector3d(0,0,x);
    end
    
    function v = rand( varargin )
      % vector of random vector3d

      if nargin < 2, varargin = [varargin 1]; end

      theta = acos(2*(rand(varargin{:})-0.5));
      rho   = 2*pi*rand(varargin{:});

      v = vector3d('theta',theta,'rho',rho);

    end
    
    function varargout = load(fname,varargin)
      % load vectors from file
      
      [varargout{1:nargout}] = loadVector3d(fname,varargin{:});
      
    end
    
    %function v = polar(polarAngle,azimuthAngle)
    %  % Syntax
    %  %
    %  x = sin(polarAngle).*cos(azimuthAngle);
    %  y = sin(polarAngle).*sin(azimuthAngle);
    %  z = cos(polarAngle);
    %  v = vector3d(x,y,z);
    %end
  end
end
